Module: Sidekiq::Worker::Overrides::ClassMethods
- Included in:
- ClassMethods
- Defined in:
- lib/sidekiq_unique_jobs/testing.rb
Overview
Prepends deletion of locks to clear
Instance Method Summary collapse
-
#clear ⇒ Object
Clears the jobs for this worker and removes all locks.
Instance Method Details
#clear ⇒ Object
Clears the jobs for this worker and removes all locks
105 106 107 108 109 110 111 |
# File 'lib/sidekiq_unique_jobs/testing.rb', line 105 def clear jobs.each do |job| SidekiqUniqueJobs::Unlockable.unlock(job) end super end |