Module: Sidekiq::Worker::SignletonOverrides
- Included in:
- Sidekiq::Worker
- Defined in:
- lib/sidekiq_unique_jobs/testing.rb
Overview
Prepends singleton methods to Sidekiq::Job
Instance Method Summary collapse
-
#clear_all ⇒ Object
Clears all jobs for this worker and removes all locks.
Instance Method Details
#clear_all ⇒ Object
Clears all jobs for this worker and removes all locks
131 132 133 134 135 |
# File 'lib/sidekiq_unique_jobs/testing.rb', line 131 def clear_all super SidekiqUniqueJobs::Digests.new.delete_by_pattern("*", count: 10_000) end |